% GENETIC_EX  And example of genetic algorithm for 1D
% (f(x)=x^2+10-10*cos(2*pi*x)) Rastrigin function. It shows every iteration
% on the plot;
function genetic_ex
% Show Rastrigin function plot
xv = -7:0.01:7;
close all;
plot(xv, rast(xv), 'b-', 'LineWidth', 2);
% Part 1. fminsearch minimization
x0 = 1.5;
x = fminsearch(@rast, x0);
hold on;
plot(x0, rast(x0), 'ko', 'LineWidth', 2);
plot(x, rast(x), 'ro', 'LineWidth', 2);
hold off;

% Part 2. Genetic algorithm
% a) Initial conditions and initial popuations
pop_size = 100;
elite_size = 10;
sigma = 1;
x0 = 25;
xpop = x0 + sigma*randn(pop_size, 1);
% b) Genetic algorithm iterations
for iter = 1:100
    % Sort the population
    [ypop, ind] = sort(rast(xpop));
    xpop = xpop(ind);
	% Draw the iteration
    plot(xv, rast(xv), 'b-', 'LineWidth', 2);
    hold on;
    plot(xpop, rast(xpop), 'bo', 'LineWidth', 2);
    xelite = xpop(1:elite_size);  
    plot(xelite, rast(xelite), 'ro', 'LineWidth', 2);
    xlabel('x', 'FontSize', 16);
    ylabel('y', 'FontSize', 16);
    title(sprintf('Iteration %d, f(%f)=%f', iter, xpop(1), ypop(1)));
    hold off;
    pause(0.5);
    drawnow;
    % Crossingover process
    w = randn(pop_size - elite_size, 1);
    ind1 = randi([1, elite_size], pop_size - elite_size, 1);
    ind2 = randi([1, elite_size], pop_size - elite_size, 1);
    xpop((elite_size+1):end) = (1 - w).*xpop(ind1) + w.*xpop(ind2);
    % Mutation process
    mind = find(rand(pop_size - elite_size, 1) < 0.2);
    xpop(mind + elite_size) = xpop(mind + elite_size) + sigma*randn(size(mind));
end
end

function y = rast(x)
% 1D Rastrigin function
y = x.^2 + 10 - 10*cos(2*pi*x);
end